%___________________________________________________________________%
%  integrated cuckoo search optimizer source codes version 1.0      %
%                                                                   %
%  Developed in MATLAB R2011a                                       %
%                                                                   %
%  Author and programmer: Xiangbo Qi                                %
%                                                                   %
%         e-Mail: drqixiangbo@gmail.com                             %
%                 ustcdragon@syu.edu.cn                             %
%                                                                   %
%                                                                   %
%                                                                   %
%   Main paper: Xiangbo Qi etal.                                    %
%              An integrated cuckoo search optimizer for single     %
%              and multi-objective optimization problems            %
%                                                                   %
%___________________________________________________________________%
function GlobalMins = func_ICSO(ProblemStruct)
runtime=ProblemStruct.Run;
maxEvaluation=ProblemStruct.Maxeva;
n=ProblemStruct.PopSize;
Dim=ProblemStruct.Dim; 
objfun=ProblemStruct.FunctionName;
func_num=ProblemStruct.Func_num;
ub=ProblemStruct.Ud;
lb=ProblemStruct.Ld;
aa=ProblemStruct.a;
bb=ProblemStruct.b;
CR=ProblemStruct.CR;
F=ProblemStruct.F;
L=ub-lb;
alphaValMax=0.01*L;
alphaValMin=0.001*L;
Ld=ones(1,Dim)*lb;
Ud=ones(1,Dim)*ub;
range=Ud-Ld;
GlobalMins=ones(runtime,maxEvaluation); 
for run=1:runtime    
    evaluation=0;     
    evaluationNow=0; 
    indStru.position=[];
    indStru.Fit=[];
    indStru.Light=[];
    initPop=repmat(indStru,n,1);
    for i=1:n
        initPop(i).position=repmat(Ld,1,1)+repmat(range,1,1).*rand(1,Dim);
        initPop(i).Fit=feval(objfun,initPop(i).position,func_num);
        initPop(i).Light=calculateFitness(initPop(i).Fit);
    end
    for tmp=1:n
        tmpFit(tmp)=initPop(tmp).Fit;
    end
    [minFit,minInd]=min(tmpFit);
    GlobalFit=minFit;
    bestCuckoo=initPop(minInd).position;    
    evaluationNow=evaluationNow+n;
    GlobalMins(run,evaluation+1:evaluationNow)=minFit;
    evaluation=evaluationNow;       
    while (evaluation < maxEvaluation)   
      tmpLight=zeros(n,1);
      for i=1:n
          tmpLight(i)=initPop(i).Light;
      end
      [descLight descInd]=sort(tmpLight,'descend');%%նȽ
      alpha=(aa-(evaluation/maxEvaluation)*(aa-bb));
      alpha=1-alpha;
      fixN=fix(alpha*n);
      %%-----------high fitness subpopulation-------------------------------      
      sunspot_pop=repmat(indStru,fixN,1);      
      for i=1:fixN
          tmpI=descInd(i);
          sunspot_pop(i).position=initPop(tmpI).position;
          sunspot_pop(i).Light=initPop(tmpI).Light;
          sunspot_pop(i).Fit=initPop(tmpI).Fit;
      end
      %%-----------low fitness subpopulation-------------------------------       
      canopy_pop=repmat(indStru,n-fixN,1);  
      j=1;
      for i=1+fixN:n
          tmpI=descInd(i);
          canopy_pop(j).position=initPop(tmpI).position;
          canopy_pop(j).Light=initPop(tmpI).Light;
          canopy_pop(j).Fit=initPop(tmpI).Fit;
          j=j+1;
      end
      %%-----------sunspot phase-------------------------------
      nestPopSub=zeros(fixN,Dim);
      ObjValnestPopSub=zeros(fixN,1);
      for i=1:fixN
          nestPopSub(i,:)=sunspot_pop(i).position;
          ObjValnestPopSub(i)=sunspot_pop(i).Fit;         
      end
      alphaVal=(alphaValMax-(evaluation/maxEvaluation)*(alphaValMax-alphaValMin));
      [nestPopSub,ObjValnestPopSub,evaluationNowSub] = sub_levy(ProblemStruct,nestPopSub,ObjValnestPopSub,alphaVal);
      evaluationNow=evaluationNow+evaluationNowSub; 
      for i=1:fixN
         sunspot_pop(i).position= nestPopSub(i,:);
         sunspot_pop(i).Fit= ObjValnestPopSub(i);         
          
      end
    nestPopSub=zeros(n-fixN,Dim);
    ObjValnestPopSub=zeros(n-fixN,1);
    
    for i=1:(n-fixN)      
            nestPopSub(i,:)=canopy_pop(i).position;
            ObjValnestPopSub(i)=canopy_pop(i).Fit;
    end
    [nestPopSub,ObjValnestPopSub,evaluationNowSub] = sub_rand(ProblemStruct,nestPopSub,ObjValnestPopSub);
    evaluationNow=evaluationNow+evaluationNowSub; 
     for i=1:(n-fixN)      
            canopy_pop(i).position=nestPopSub(i,:);
            canopy_pop(i).Fit=ObjValnestPopSub(i);
     end
     initPop=[sunspot_pop;canopy_pop];
     %%%%%crossover
     for i=1:n
         Param2Change=fix(rand*Dim)+1;            
         neighbour=fix(rand*(n))+1;
         for tmp=1:n
             tmpFit(tmp)=initPop(tmp).Fit;
         end
         [minFit,minInd]=min(tmpFit);
         g=minInd(end);
         while((neighbour==i)||(neighbour==g))
                neighbour=fix(rand*n)+1;
         end;
         a=neighbour;
         b=fix(rand*n)+1;
         while((b==i)||(b==a)||(b==g))
                b=fix(rand*n)+1;
         end;
         c=fix(rand*(n))+1;
         while((c==i)||(c==a)||(c==b)||(c==g))
                c=fix(rand*(n))+1;
         end;
         k=fix(rand*(n))+1;
         while((k==i)||(k==a)||(k==b)||(k==c)||(k==g))
                k=fix(rand*(n))+1;
         end;
         sol=initPop(i).position;
         for d=1:Dim
             if((rand()<CR)||(d==Dim))
                cSol=initPop(c).position;
                aSol=initPop(a).position;
                bSol=initPop(b).position;
                sol(Param2Change)=cSol(Param2Change)+F*(aSol(Param2Change)-bSol(Param2Change));
                  
              end
                Param2Change=fix(rand*Dim)+1;
          end
          ind=find(sol<Ld);
          sol(ind)=Ld(ind);
          ind=find(sol>Ud);
          sol(ind)=Ud(ind);
          ObjValSol=feval(objfun,sol,func_num);
          evaluationNow=evaluationNow+1;
          if(ObjValSol<initPop(i).Fit)
                initPop(i).position=sol;
                initPop(i).Fit=ObjValSol;
          end;
     end;
     for tmp=1:n
            tmpFit(tmp)=initPop(tmp).Fit;
     end
     [minFit,minInd]=min(tmpFit);
     bestCuckoo=initPop(minInd).position;
     if(minFit<GlobalFit)    
            GlobalFit=minFit;
     end
     if(evaluationNow<maxEvaluation)    
            GlobalMins(run,evaluation+1:evaluationNow)=GlobalFit;
            evaluation=evaluationNow;
     else
           GlobalMins(run,evaluation+1:maxEvaluation)=GlobalFit;
           break;
     end
  end
  fprintf('runs=%d ObjVal=%g\n',run,minFit);
end
end










